import React from 'react'
import * as d3 from 'd3'
import PropTypes from 'prop-types'
import * as utils from './TreeMap.utils'

class TreeMap extends React.PureComponent {
  static propTypes = {
    data: PropTypes.object.isRequired,
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    onClick: PropTypes.func
  }

  state = {
    data: this.props.data
  }

  get size() {
    const { width, height } = this.props
    return [width, height]
  }

  get root() {
    const { data } = this.state

    const treemap = d3
      .treemap()
      .tile(d3.treemapBinary)
      .size(this.size)
      .round(true)
      .paddingInner(4)
      .paddingOuter(1)

    const root = d3
      .hierarchy(data)
      .eachBefore(utils.eachBefore)
      .sum(utils.sumBySize)
      .sort(utils.sort)

    treemap(root)

    const array = root.descendants().reverse()
    return array.splice(0, array.length - 1)
  }

  setSvg = svg => {
    if (svg) {
      d3.select(svg)
        .selectAll('g')
        .remove()
      this.draw(svg)
    }
  }

  restore = () => {
    this.setState({
      data: this.props.data
    })
  }

  draw = svg => {
    const { onClick } = this.props

    const cell = d3
      .select(svg)
      .selectAll('g')
      .data(this.root)
      .enter()
      .append('g')
      .attr('transform', utils.getTransform)

    cell
      .append('rect')
      .attr('id', utils.getId)
      .attr('width', utils.getWidth)
      .attr('height', utils.getHeight)
      .attr('fill', utils.setColor)

    cell
      .append('clipPath')
      .attr('id', utils.getClipId)
      .append('use')
      .attr('xlink:href', utils.getHrefId)

    cell
      .append('foreignObject')
      .attr('clip-path', utils.getUrlId)
      .attr('width', utils.getWidth)
      .attr('height', utils.getHeight)
      .append('xhtml:div')
      .attr('style', utils.getBlockStyle)
      .append('xhtml:p')
      .html(utils.getName)
      .attr('style', utils.getParagraphStyle)
      .append('xhtml:span')
      .html(utils.getRate)

    cell.append('title').text(utils.getTitle)

    cell.on('click', d => {
      if (d.children) {
        this.setState({
          data: d.data
        })
      } else if (onClick) {
        onClick(d)
      }
    })
  }

  render() {
    const { data } = this.state
    const { width, height } = this.props

    return (
      <svg key={data.name} ref={this.setSvg} width={width} height={height} />
    )
  }
}

export default TreeMap
